@@ -4,8 +4,8 @@ from __future__ import absolute_import, unicode_literals
 # But the code fully another... It's not a fork anymore...
 import sys
 import copy
-from functools import partial
 import warnings
+from functools import partial, wraps
 
 try:
     str = unicode  # Python 2.* compatible
@@ -23,9 +23,6 @@ LOOKUP_SEP = '__'
 
 
 class SqlDialects(object):
-    """
-    Stores all dialect representations
-    """
 
     __slots__ = ('_registry', )
 
@@ -33,41 +30,38 @@ class SqlDialects(object):
         self._registry = {}
 
     def register(self, dialect, cls):
-        def decorator(func):
-            ns = self._registry.setdefault(dialect, {})
-            ns[cls] = func
+        def deco(func):
+            self._registry.setdefault(dialect, {})[cls] = func
             return func
-        return decorator
+        return deco
 
     def sqlrepr(self, dialect, cls):
         ns = self._registry.setdefault(dialect, {})
         for t in cls.mro():
-            if t in ns:  # Looking for registered dialect
-                return ns[t]
-            elif '__sqlrepr__' in t.__dict__:  # Looking for __sqlrepr__ directly in class, except parent classes
-                return t.__sqlrepr__
+            r = ns.get(t, t.__dict__.get('__sqlrepr__'))
+            if r:
+                return r
         return None
 
 sql_dialects = SqlDialects()
 
 
 def opt_checker(k_list):
-    def new_deco(func):
+    def new_deco(f):
+        @wraps(f)
         def new_func(self, *args, **opt):
             for k, v in list(opt.items()):
                 if k not in k_list:
                     raise TypeError("Not implemented option: {0}".format(k))
-            return func(self, *args, **opt)
-
-        new_func.__doc__ = func.__doc__
+            return f(self, *args, **opt)
         return new_func
     return new_deco
 
 
 def same(name):
-    def deco(self, *a, **kw):
+    def f(self, *a, **kw):
         return getattr(self, name)(*a, **kw)
-    return deco
+    return f
 
 
 class Error(Exception):
@@ -87,6 +81,21 @@ class Comparable(object):
     def _p(op):
         return lambda self: Prefix(op, self)
 
+    def _l(mask, ci=False, inv=False):
+        a = 'like'
+        if ci:
+            a = 'i' + a
+        if inv:
+            a = 'r' + a
+        def f(self, other):
+            args = [other]
+            if 4 & mask:
+                args.insert(0, '%')
+            if 1 & mask:
+                args.append('%')
+            return getattr(self, a)(Concat(*args))
+        return f
+
     __add__ = _c("+")
     __radd__ = _c("+", 1)
     __sub__ = _c("-")
@@ -105,6 +114,8 @@ class Comparable(object):
     __le__ = _c("<=")
     like = _c("LIKE")
     ilike = _c("ILIKE")
+    rlike = _c("LIKE", 1)
+    rilike = _c("ILIKE", 1)
 
     __pos__ = _p("+")
     __neg__ = _p("-")
@@ -118,6 +129,19 @@ class Comparable(object):
     __abs__ = _ca("ABS")
     count = _ca("COUNT")
 
+    startswith = _l(1)
+    istartswith = _l(1, 1)
+    contains = _l(5)
+    icontains = _l(5, 1)
+    endswith = _l(4)
+    iendswith = _l(4, 1)
+    rstartswith = _l(1, 0, 1)
+    ristartswith = _l(1, 1, 1)
+    rcontains = _l(5, 0, 1)
+    ricontains = _l(5, 1, 1)
+    rendswith = _l(4, 0, 1)
+    riendswith = _l(4, 1, 1)
+
     def __eq__(self, other):
         if other is None:
             return Condition("IS", self, Constant("NULL"))
@@ -149,24 +173,6 @@ class Comparable(object):
             other = ExprList(*other).join(", ")
         return ExprList(self, Constant("NOT IN"), Parentheses(other)).join(" ")
 
-    def startswith(self, other):
-        return self.like(Concat(other, '%'))
-
-    def istartswith(self, other):
-        return self.ilike(Concat(other, '%'))
-
-    def contains(self, other):
-        return self.like(Concat('%', other, '%'))
-
-    def icontains(self, other):
-        return self.ilike(Concat('%', other, '%'))
-
-    def endswith(self, other):
-        return self.like(Concat('%', other))
-
-    def iendswith(self, other):
-        return self.ilike(Concat('%', other))
-
     def between(self, start, end):
         return Between(self, start, end)
 
@@ -193,13 +199,6 @@ class Comparable(object):
 
     __hash__ = None
 
-    AS = same('as_')
-    IN = same('in_')
-    NOT_IN = same('not_in')
-    LIKE = same('like')
-    ILIKE = same('ilike')
-    BETWEEN = same('between')
-
 
 class Expr(Comparable):
 
@@ -300,16 +299,15 @@ class ExprList(Expr):
 
     def __sqlrepr__(self, dialect):
         sqls = []
-        for arg in self._args:
-            sql = sqlrepr(arg, dialect)
-            # Some actions here if need
+        for a in self._args:
+            sql = sqlrepr(a, dialect)
             sqls.append(sql)
         return self._sep.join(sqls)
 
     def __params__(self):
         params = []
-        for arg in self._args:
-            params.extend(sqlparams(arg))
+        for a in self._args:
+            params.extend(sqlparams(a))
         return params
 
     def __copy__(self):
@@ -318,6 +316,24 @@ class ExprList(Expr):
         return dup
 
 
+class FieldList(ExprList):
+
+    __slots__ = ()
+
+    def _build(self):
+        sql = ExprList().join(self._sep)
+        for a in self._args:
+            if isinstance(a, Alias):
+                a = ExprList(a._expr, Constant("AS"), a).join(" ")
+            sql.append(a)
+        return sql
+
+    def __sqlrepr__(self, dialect):
+        return sqlrepr(self._build(), dialect)
+
+    def __params__(self):
+        return sqlparams(self._build())
+
 class Concat(ExprList):
 
     __slots__ = ('_sep', '_args', '_ws')
@@ -332,10 +348,7 @@ class Concat(ExprList):
         return self
 
     def __params__(self):
-        params = []
-        params.extend(sqlparams(self._ws))
-        params.extend(super(Concat, self).__params__())
-        return params
+        return sqlparams(self._ws) + super(Concat, self).__params__()
 
     def __sqlrepr__(self, dialect):
         value = super(Concat, self).__sqlrepr__(dialect)
@@ -366,6 +379,11 @@ class Parentheses(Expr):
         return sqlparams(self._expr)
 
 
+class OmitParentheses(Parentheses):
+    def __sqlrepr__(self, dialect):
+        return sqlrepr(self._expr, dialect)
+
+
 class Prefix(Expr):
 
     __slots__ = ('_prefix', '_expr', )
@@ -406,12 +424,7 @@ class Between(Expr):
         self._end = prepare_expr(end)
 
     def __sqlrepr__(self, dialect):
-        sqls = [
-            sqlrepr(self._expr, dialect),
-            sqlrepr(self._start, dialect),
-            sqlrepr(self._end, dialect),
-        ]
-        return "{0} BETWEEN {1} AND {2}".format(*sqls)
+        return "{0} BETWEEN {1} AND {2}".format(sqlrepr(self._expr, dialect), sqlrepr(self._start, dialect), sqlrepr(self._end, dialect))
 
     def __params__(self):
         return sqlparams(self._expr) + sqlparams(self._start) + sqlparams(self._end)
@@ -451,8 +464,6 @@ class ConstantSpace(object):
     __slots__ = ()
 
     def __getattr__(self, attr):
-        if attr.startswith('__'):
-            raise AttributeError
         return Constant(attr)
 
 
@@ -462,16 +473,9 @@ class MetaField(type):
         if key[0] == '_':
             raise AttributeError
         parts = key.split(LOOKUP_SEP, 2)
-        name = parts[0]
-        prefix = None
-        alias = None
-
-        if len(parts) > 1:
-            prefix = parts[0]
-            name = parts[1]
-        if len(parts) > 2:
-            alias = parts[2]
-
+        prefix, name, alias = parts + [None] * (3 - len(parts))
+        if name is None:
+            prefix, name = name, prefix
         f = cls(name, prefix)
         if alias is not None:
             f = f.as_(alias)
@@ -504,25 +508,17 @@ class Alias(Expr):
         self._expr = expr
         super(Alias, self).__init__(alias)
 
-    @property
-    def expr(self):
-        return self._expr
-
     def __sqlrepr__(self, dialect):
         return qn(self._sql, dialect)
 
 
-class Index(Alias):
-    pass
-
-
 class MetaTable(type):
 
     def __new__(cls, name, bases, attrs):
         def _f(attr):
             return lambda self, *a, **kw: getattr(TableJoin(self), attr)(*a, **kw)
 
-        for a in ['inner_join', 'left_join', 'right_join', 'full_join', 'cross_join', 'join', 'on', 'use_index', 'ignore_index', 'force_index']:
+        for a in ['inner_join', 'left_join', 'right_join', 'full_join', 'cross_join', 'join', 'on', 'hint']:
             attrs[a] = _f(a)
         return type.__new__(cls, name, bases, attrs)
 
@@ -530,12 +526,7 @@ class MetaTable(type):
         if key[0] == '_':
             raise AttributeError
         parts = key.split(LOOKUP_SEP, 1)
-        name = parts[0]
-        alias = None
-
-        if len(parts) > 1:
-            alias = parts[1]
-
+        name, alias = parts + [None] * (2 - len(parts))
         table = cls(name)
         if alias is not None:
             table = table.as_(alias)
@@ -556,8 +547,7 @@ class Table(MetaTable(bytes("NewBase"), (object, ), {})):
         if name[0] == '_':
             raise AttributeError
         parts = name.split(LOOKUP_SEP, 1)
-        name = parts.pop(0)
-        alias = parts.pop(0) if len(parts) else None
+        name, alias = parts + [None] * (2 - len(parts))
         f = Field(name, self)
         if alias is not None:
             f = f.as_(alias)
@@ -574,11 +564,6 @@ class Table(MetaTable(bytes("NewBase"), (object, ), {})):
     __sub__ = same('right_join')
     __or__ = same('full_join')
     __mul__ = same('cross_join')
-    AS = same('as_')
-    ON = same('on')
-    USE_INDEX = same('use_index')
-    IGNORE_INDEX = same('ignore_index')
-    FORCE_INDEX = same('force_index')
 
 
 class TableAlias(Table):
@@ -589,10 +574,6 @@ class TableAlias(Table):
         self._table = table
         self._alias = alias
 
-    @property
-    def table(self):
-        return self._table
-
     def as_(self, alias):
         return type(self)(alias, self._table)
 
@@ -602,11 +583,11 @@ class TableAlias(Table):
 
 class TableJoin(object):
 
-    __slots__ = ('_table', '_alias', '_join_type', '_on', '_left', '_use_index', '_ignore_index', '_force_index', )
+    __slots__ = ('_table', '_alias', '_join_type', '_on', '_left', '_hint', )
 
     def __init__(self, table_or_alias, join_type=None, on=None, left=None):
         if isinstance(table_or_alias, TableAlias):
-            self._table = table_or_alias.table
+            self._table = table_or_alias._table
             self._alias = table_or_alias
         else:
             self._table = table_or_alias
@@ -614,9 +595,7 @@ class TableJoin(object):
         self._join_type = join_type
         self._on = on and parentheses_conditional(on) or on
         self._left = left
-        self._use_index = ExprList().join(", ")
-        self._ignore_index = ExprList().join(", ")
-        self._force_index = ExprList().join(", ")
+        self._hint = None
 
     def _j(j):
         return lambda self, obj: self.join(j, obj)
@@ -652,34 +631,15 @@ class TableJoin(object):
     def group(self):
         return TableJoin(self)
 
-    @opt_checker(["reset", ])
-    def change_index(self, index, *args, **opts):
-        if opts.get("reset"):
-            index.reset()
-        args = list(args)
-        if len(args):
-            if hasattr(args[0], '__iter__'):
-                self = self.change_index(index, *args.pop(0), reset=True)
-            if len(args):
-                for i, arg in enumerate(args):
-                    if isinstance(arg, string_types):
-                        args[i] = Index(arg, self._table)
-                index.extend(args)
-                return self
+    def hint(self, expr):
+        if isinstance(expr, string_types):
+            expr = Expr(expr)
+        self._hint = OmitParentheses(expr)
         return self
 
-    def use_index(self, *args, **opts):
-        return self.change_index(self._use_index, *args, **opts)
-
-    def ignore_index(self, *args, **opts):
-        return self.change_index(self._ignore_index, *args, **opts)
-
-    def force_index(self, *args, **opts):
-        return self.change_index(self._force_index, *args, **opts)
-
     def __copy__(self):
         dup = copy.copy(super(TableJoin, self))
-        for a in ['_use_index', '_ignore_index', '_force_index', ]:
+        for a in ['_hint', ]:
             setattr(dup, a, copy.copy(getattr(dup, a, None)))
         return dup
 
@@ -695,19 +655,14 @@ class TableJoin(object):
             sql.append(self._table)
         if self._alias is not None:
             sql.extend([Constant("AS"), self._alias])
-        if dialect in ('mysql', ):
-            if self._use_index:
-                sql.extend([Constant("USE INDEX"), Parentheses(self._use_index)])
-            if self._ignore_index:
-                sql.extend([Constant("USE INDEX"), Parentheses(self._ignore_index)])
-            if self._force_index:
-                sql.extend([Constant("USE INDEX"), Parentheses(self._force_index)])
         if self._on is not None:
             sql.extend([Constant("ON"), self._on])
+        if self._hint is not None:
+            sql.append(self._hint)
         return sqlrepr(sql, dialect)
 
     def __params__(self):
-        return sqlparams(self._left) + sqlparams(self._table) + sqlparams(self._on)
+        return sqlparams(self._left) + sqlparams(self._table) + sqlparams(self._on) + sqlparams(self._hint)
 
     as_nested = same('group')
     __and__ = same('inner_join')
@@ -715,18 +670,25 @@ class TableJoin(object):
     __sub__ = same('right_join')
     __or__ = same('full_join')
     __mul__ = same('cross_join')
-    ON = same('on')
-    USE_INDEX = same('use_index')
-    IGNORE_INDEX = same('ignore_index')
-    FORCE_INDEX = same('force_index')
 
 
 class QuerySet(Expr):
 
+    _clauses = (
+        ('fields', None, '_fields'),
+        ('tables', None, '_tables'),
+        ('from', 'FROM', '_tables'),
+        ('where', 'WHERE', '_wheres'),
+        ('group', 'GROUP BY', '_group_by'),
+        ('having', 'HAVING', '_havings'),
+        ('order', 'ORDER BY', '_order_by'),
+        ('limit', None, '_limit')
+    )
+
     def __init__(self, tables=None):
 
         self._distinct = False
-        self._fields = ExprList().join(", ")
+        self._fields = FieldList().join(", ")
         if tables:
             if not isinstance(tables, TableJoin):
                 tables = TableJoin(tables)
@@ -748,24 +710,7 @@ class QuerySet(Expr):
         self._sql = None
         self._params = []
 
-    @property
-    def wheres(self):
-        return self._wheres
-
-    @wheres.setter
-    def wheres(self, cs):
-        self._wheres = cs
-
-    @property
-    def havings(self):
-        return self._havings
-
-    @havings.setter
-    def havings(self, cs):
-        self._havings = cs
-
     def clone(self):
-        # return copy.deepcopy(self)
         dup = copy.copy(super(QuerySet, self))
         for a in ['_fields', '_tables', '_group_by', '_order_by', '_values', '_key_values', ]:
             setattr(dup, a, copy.copy(getattr(dup, a, None)))
@@ -816,6 +761,7 @@ class QuerySet(Expr):
         return self._fields
 
     def on(self, c):
+        # TODO: Remove?
         self = self.clone()
         if not isinstance(self._tables, TableJoin):
             raise Error("Can't set on without join table")
@@ -824,18 +770,12 @@ class QuerySet(Expr):
 
     def where(self, c):
         self = self.clone()
-        if self._wheres is None:
-            self._wheres = c
-        else:
-            self.wheres = self.wheres & c
+        self._wheres = c if self._wheres is None else self._wheres & c
         return self
 
     def or_where(self, c):
         self = self.clone()
-        if self._wheres is None:
-            self._wheres = c
-        else:
-            self.wheres = self.wheres | c
+        self._wheres = c if self._wheres is None else self._wheres | c
         return self
 
     @opt_checker(["reset", ])
@@ -857,18 +797,12 @@ class QuerySet(Expr):
 
     def having(self, c):
         self = self.clone()
-        if self._havings is None:
-            self._havings = c
-        else:
-            self.havings = self.havings & c
+        self._havings = c if self._havings is None else self._havings & c
         return self
 
     def or_having(self, c):
         self = self.clone()
-        if self._havings is None:
-            self._havings = c
-        else:
-            self.havings = self.havings | c
+        self._havings = c if self._havings is None else self._havings | c
         return self
 
     @opt_checker(["desc", "reset", ])
@@ -894,23 +828,13 @@ class QuerySet(Expr):
 
     def limit(self, *args, **kwargs):
         self = self.clone()
-        limit = None
-        offset = 0
-
-        if len(args) == 1:
-            limit = args[0]
-        elif len(args) == 2:
-            offset = args[0]
-            limit = args[1]
-        if len(args) > 2:
-            raise Error("Too many arguments for limit.")
-
-        if len(args) == 0:
-            if 'limit' in kwargs:
-                limit = kwargs['limit']
-            if 'offset' in kwargs:
-                offset = kwargs['offset']
-
+        if args:
+            if len(args) < 2:
+                args = (0,) + args
+            offset, limit = args
+        else:
+            limit = kwargs.get('limit')
+            offset = kwargs.get('offset', 0)
         sql = ""
         if limit:
             sql = "LIMIT {0:d}".format(limit)
@@ -920,36 +844,13 @@ class QuerySet(Expr):
         return self
 
     def __getitem__(self, key):
-        """Returns self.limit()"""
-        offset = 0
-        limit = None
         if isinstance(key, slice):
-            if key.start is not None:
-                offset = int(key.start)
-            if key.stop is not None:
-                end = int(key.stop)
-                limit = end - offset
+            offset = key.start or 0
+            limit = key.stop - offset if key.stop else None
         else:
-            offset = key
-            limit = 1
+            offset, limit = key, 1
         return self.limit(offset, limit)
 
-    @opt_checker(["distinct", "for_update"])
-    def count(self, *args, **opts):
-        self = self.clone()
-        self._action = "count"
-        if len(args):
-            self = self.fields(*args)
-        if opts.get("distinct"):
-            self = self.distinct(True)
-        if opts.get("for_update"):
-            self._for_update = True
-        return self.result()
-
-    @opt_checker(["distinct", "for_update"])
-    def select_one(self, *args, **opts):
-        return self.limit(1).select(*args, **opts)
-
     @opt_checker(["distinct", "for_update"])
     def select(self, *args, **opts):
         self = self.clone()
@@ -964,11 +865,7 @@ class QuerySet(Expr):
 
     def insert(self, fv_dict, **opts):
         items = list(fv_dict.items())
-        return self.insert_many(
-            [x[0] for x in items],
-            ([x[1] for x in items], ),
-            **opts
-        )
+        return self.insert_many([x[0] for x in items], ([x[1] for x in items], ), **opts)
 
     @opt_checker(["ignore", "on_duplicate_key_update"])
     def insert_many(self, fields, values, **opts):
@@ -988,7 +885,7 @@ class QuerySet(Expr):
             for f, v in opts.get("on_duplicate_key_update").items():
                 if not isinstance(f, Expr):
                     f = Field(f)
-                self._on_duplicate_key_update.append(ExprList(f, Constant("=="), v))
+                self._on_duplicate_key_update.append(ExprList(f, Constant("="), v))
         return self.result()
 
     @opt_checker(["ignore"])
@@ -1022,27 +919,11 @@ class QuerySet(Expr):
         return self.execute()
 
     def _sql_extend(self, sql, parts):
-        if "fields" in parts and self._fields:
-            fields = ExprList().join(", ")
-            for f in self._fields:
-                if isinstance(f, Alias):
-                    f = ExprList(f.expr, Constant("AS"), f).join(" ")
-                fields.append(f)
-            sql.append(fields)
-        if "tables" in parts and self._tables:
-            sql.append(self._tables)
-        if "from" in parts and self._tables:
-            sql.extend([Constant("FROM"), self._tables])
-        if "where" in parts and self._wheres:
-            sql.extend([Constant("WHERE"), self._wheres])
-        if "group" in parts and self._group_by:
-            sql.extend([Constant("GROUP BY"), self._group_by])
-        if "having" in parts and self._havings:
-            sql.extend([Constant("HAVING"), self._havings])
-        if "order" in parts and self._order_by:
-            sql.extend([Constant("ORDER BY"), self._order_by])
-        if "limit" in parts and self._limit:
-            sql.append(self._limit)
+        for key, clause, attr in self._clauses:
+            if key in parts and getattr(self, attr):
+                if clause:
+                    sql.append(Constant(clause))
+                sql.append(getattr(self, attr))
 
     def _build_sql(self):
         sql = ExprList().join(" ")
@@ -1055,22 +936,6 @@ class QuerySet(Expr):
             if self._for_update:
                 sql.append(Constant("FOR UPDATE"))
 
-        elif self._action == "count":
-            sql.append(Constant("SELECT"))
-            count_distinct = self._distinct
-            fields = self._fields
-            if len(fields) == 0:
-                fields = self._group_by
-                count_distinct = True
-            if count_distinct:
-                fields = ExprList(Constant("COUNT")(Prefix("DISTINCT", fields))).join(", ")
-            else:
-                fields = ExprList(Constant("COUNT")(fields)).join(", ")
-            sql.append(fields)
-            self._sql_extend(sql, ["from", "where", ])
-            if self._for_update:
-                sql.append(Constant("FOR UPDATE"))
-
         elif self._action == "insert":
             sql.append(Constant("INSERT"))
             if self._ignore:
@@ -1170,9 +1035,7 @@ def placeholder_conditional(expr):
 
 
 def parentheses_conditional(expr):
-    if isinstance(expr, (Condition, QuerySet)):
-        return Parentheses(expr)
-    if expr.__class__ == Expr:
+    if isinstance(expr, (Condition, QuerySet)) or type(expr) == Expr:
         return Parentheses(expr)
     return expr
 
@@ -1192,8 +1055,7 @@ def default_dialect(dialect=None):
 
 def sqlrepr(obj, dialect=None):
     """Renders query set"""
-    if dialect is None:
-        dialect = DEFAULT_DIALECT
+    dialect = dialect or DEFAULT_DIALECT
     callback = sql_dialects.sqlrepr(dialect, obj.__class__)
     if callback is not None:
         return callback(obj, dialect)
@@ -1201,22 +1063,17 @@ def sqlrepr(obj, dialect=None):
 
 
 def sqlparams(obj):
-    """Renders query set"""
+    """Returns query set params"""
     if hasattr(obj, '__params__'):
         return list(obj.__params__())
     return []
 
 
 def warn(old, new, stacklevel=3):
-    warnings.warn(
-        "{0} is deprecated. Use {1} instead".format(old, new),
-        PendingDeprecationWarning,
-        stacklevel=stacklevel
-    )
+    warnings.warn("{0} is deprecated. Use {1} instead".format(old, new), PendingDeprecationWarning, stacklevel=stacklevel)
 
 T, TA, F, A, E, QS = Table, TableAlias, Field, Alias, Expr, QuerySet
-const = ConstantSpace()
-func = const
+func = const = ConstantSpace()
 qn = Name()
 
 for cls in (Expr, Table, TableJoin, ):
