@@ -47,33 +47,13 @@ class TestSmartSQL(unittest.TestCase):
             ('SELECT "t1"."id" FROM ("t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id")) INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")', [], )
         )
 
-    def test_index(self):
-        q = QS(T.tb.use_index('index1')).dialect('mysql')
-        self.assertEqual(
-            q.select('*'),
-            ('SELECT * FROM `tb` USE INDEX (`index1`)', [], )
-        )
-        q = q.tables(q.tables().use_index('index2'))
-        self.assertEqual(
-            q.select('*'),
-            ('SELECT * FROM `tb` USE INDEX (`index1`, `index2`)', [], )
-        )
-        q = q.tables(q.tables().use_index(['index3', 'index4']))
-        self.assertEqual(
-            q.select('*'),
-            ('SELECT * FROM `tb` USE INDEX (`index3`, `index4`)', [])
-        )
-        q = q.tables(q.tables().use_index('index5', 'index6', reset=True))
-        self.assertEqual(
-            q.select('*'),
-            ('SELECT * FROM `tb` USE INDEX (`index5`, `index6`)', [])
-        )
+    def test_hint(self):
         t1 = T.tb1
         t2 = T.tb1.as_('al2')
-        q = QS(t1 & t2.use_index('index1', 'index2').on(t2.parent_id == t1.id)).dialect('mysql')
+        q = QS(t1 & t2.hint(E('USE INDEX (`index1`, `index2`)')).on(t2.parent_id == t1.id)).dialect('mysql')
         self.assertEqual(
             q.select(t2.id),
-            ('SELECT `al2`.`id` FROM `tb1` INNER JOIN `tb1` AS `al2` USE INDEX (`index1`, `index2`) ON (`al2`.`parent_id` = `tb1`.`id`)', [], )
+            ('SELECT `al2`.`id` FROM `tb1` INNER JOIN `tb1` AS `al2` ON (`al2`.`parent_id` = `tb1`.`id`) USE INDEX (`index1`, `index2`)', [], )
         )
 
     def test_prefix(self):
@@ -257,14 +237,10 @@ class TestSmartSQL(unittest.TestCase):
             ('SELECT "grade"."name" FROM "grade" INNER JOIN "base" ON ("grade"."item_type" = "base"."type") LEFT OUTER JOIN "lottery" ON ("base"."type" = "lottery"."item_type") WHERE ("base"."type" = %s) FOR UPDATE', [1, ], )
         )
         w = w & (F.grade__status == [0, 1])
-        self.assertEqual(
-            QS(t).where(w).group_by(F.grade__name, F.base__img).count(),
-            ('SELECT COUNT(DISTINCT "grade"."name", "base"."img") FROM "grade" INNER JOIN "base" ON ("grade"."item_type" = "base"."type") LEFT OUTER JOIN "lottery" ON ("base"."type" = "lottery"."item_type") WHERE (("base"."type" = %s) AND "grade"."status" IN (%s, %s))', [1, 0, 1, ], )
-        )
         now = datetime.datetime.now()
         w = w | (F.lottery__add_time > "2009-01-01") & (F.lottery__add_time <= now)
         self.assertEqual(
-            QS(t).where(w).select_one(F.grade__name, F.base__img, F.lottery__price),
+            QS(t).where(w).limit(1).select(F.grade__name, F.base__img, F.lottery__price),
             ('SELECT "grade"."name", "base"."img", "lottery"."price" FROM "grade" INNER JOIN "base" ON ("grade"."item_type" = "base"."type") LEFT OUTER JOIN "lottery" ON ("base"."type" = "lottery"."item_type") WHERE ((("base"."type" = %s) AND "grade"."status" IN (%s, %s)) OR (("lottery"."add_time" > %s) AND ("lottery"."add_time" <= %s))) LIMIT 1', [1, 0, 1, '2009-01-01', now, ], )
         )
         w = w & (F.base__status != [1, 2])
@@ -286,15 +262,15 @@ class TestSmartSQL(unittest.TestCase):
             qs.select(F.user__name, F.address__street),
             ('SELECT "user"."name", "address"."street" FROM "user" INNER JOIN "address" ON ("user"."id" = "address"."user_id")', [], )
         )
-        qs.wheres = qs.wheres & (F.id == 1)
+        qs = qs.where(F.id == 1)
         self.assertEqual(
             qs.select(F.name, F.id),
-            ('SELECT "name", "id" FROM "user" INNER JOIN "address" ON ("user"."id" = "address"."user_id") WHERE (("id" = %s))', [1, ], )
+            ('SELECT "name", "id" FROM "user" INNER JOIN "address" ON ("user"."id" = "address"."user_id") WHERE ("id" = %s)', [1, ], )
         )
-        qs.wheres = qs.wheres & ((F.address__city_id == [111, 112]) | E("address.city_id IS NULL"))
+        qs = qs.where((F.address__city_id == [111, 112]) | E("address.city_id IS NULL"))
         self.assertEqual(
             qs.select(F.user__name, F.address__street, func.COUNT(Constant("*")).as_("count")),
-            ('SELECT "user"."name", "address"."street", COUNT(*) AS "count" FROM "user" INNER JOIN "address" ON ("user"."id" = "address"."user_id") WHERE ((("id" = %s)) AND ("address"."city_id" IN (%s, %s) OR (address.city_id IS NULL)))', [1, 111, 112, ], )
+            ('SELECT "user"."name", "address"."street", COUNT(*) AS "count" FROM "user" INNER JOIN "address" ON ("user"."id" = "address"."user_id") WHERE (("id" = %s) AND ("address"."city_id" IN (%s, %s) OR (address.city_id IS NULL)))', [1, 111, 112, ], )
         )
 
     def test_subquery(self):
@@ -343,7 +319,7 @@ class TestSmartSQL(unittest.TestCase):
         vl = (("garfield", "male", 0, 1), ("superwoman", "female", 0, 10))
         self.assertEqual(
             QS(T.user).insert_many(fl, vl, on_duplicate_key_update={"age": E("age + VALUES(age)")}),
-            ('INSERT INTO "user" ("name", "gender", "status", "age") VALUES (%s, %s, %s, %s) (%s, %s, %s, %s) ON DUPLICATE KEY UPDATE "age" == (age + VALUES(age))', ['garfield', 'male', 0, 1, 'superwoman', 'female', 0, 10, ], )
+            ('INSERT INTO "user" ("name", "gender", "status", "age") VALUES (%s, %s, %s, %s) (%s, %s, %s, %s) ON DUPLICATE KEY UPDATE "age" = (age + VALUES(age))', ['garfield', 'male', 0, 1, 'superwoman', 'female', 0, 10, ], )
         )
 
     def test_update(self):
